/*
*
* ====================================================================
*
* Copyright 2004 The Apache Software Foundation 
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
*/
package org.apache.commons.i18n;

import java.io.InputStream;
import java.text.MessageFormat;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.helpers.DefaultHandler;

/**
 * @author Daniel Florey
 *
 */
public class XMLMessageProvider implements MessageProvider {
    private static Logger logger = Logger.getLogger(XMLMessageProvider.class.getName());

    private static SAXParserFactory factory = SAXParserFactory.newInstance();
    
    private static Map installedMessages = new HashMap();
    private static Map messages = new HashMap();
    
    public String getText(String id, String entry, Locale locale) throws MessageNotFoundException {
        Message message = findMessage(id, locale);
        return message.getEntry(entry);
    }

    public Map getEntries(String id, Locale locale) throws MessageNotFoundException {
        Message message = findMessage(id, locale);
        return message.getEntries();
    }

    public static void install(String id, InputStream inputStream) {
        try {
            Map applicationMessages = new HashMap();
            SAXParser parser = factory.newSAXParser();
            ConfigurationHandler handler = new ConfigurationHandler();
            parser.parse(new InputSource(inputStream), handler);
            Map parsedMessages = handler.getMessages();
            applicationMessages.putAll(parsedMessages);
            messages.putAll(applicationMessages);
            installedMessages.put(id, applicationMessages.keySet());
        } catch (Exception exception) {
            logger.log(Level.SEVERE, 
                    MessageFormat.format(
                    MessageManager.INTERNAL_MESSAGES.getString(MessageManager.MESSAGE_PARSING_ERROR),
                    new String[] { id }), exception); 
        }
    }
    
    public static void uninstall(String id) {
        Collection messageKeys = (Collection)installedMessages.get(id);
        for ( Iterator i = messageKeys.iterator(); i.hasNext(); ) {
            String messageKey = (String)i.next();
            messages.remove(messageKey);
        }
        installedMessages.remove(id);
    }
    
    public static void update(String id, InputStream inputStream) {
        uninstall(id);
        install(id, inputStream);
    }
        
    private static Message findMessage(String id, Locale locale) {
        Message message = lookupMessage(id, locale);
        if (message == null) {
            message = lookupMessage(id, Locale.getDefault());
        }
        if (message == null ) throw new MessageNotFoundException(
                MessageFormat.format(
                        MessageManager.INTERNAL_MESSAGES.getString(MessageManager.MESSAGE_NOT_FOUND),
                        new String[] { id })); 
        return message;
    }

    private static Message lookupMessage(String id, Locale locale) {
        StringBuffer keyBuffer = new StringBuffer(64);
        keyBuffer.append(id);
        if (locale.getLanguage() != null) keyBuffer.append("_" + locale.getLanguage());
        if (locale.getCountry() != null) keyBuffer.append("_" + locale.getCountry());
        if (locale.getVariant() != null) keyBuffer.append("_" + locale.getVariant());
        String key = keyBuffer.toString();
        if (messages.containsKey(key)) return (Message)messages.get(key);
        while (key.lastIndexOf('_') > 0) {
            key = key.substring(0, key.lastIndexOf('_'));
            if (messages.containsKey(key)) return (Message)messages.get(key);
        }
        return null;
    }

    static class ConfigurationHandler extends DefaultHandler {
        private Map messages = new HashMap();
        private String id, key;
        private Message message;
        private StringBuffer cData;

        public void startElement(String namespaceUri, String localeName, String qName, Attributes attributes)  {
            if (qName.matches("message")) {
                id = attributes.getValue("id");
            } else if (qName.matches("locale")) {
                message = new Message(id);
                message.setLanguage(attributes.getValue("language"));
                message.setCountry(attributes.getValue("country"));
                message.setVariant(attributes.getValue("variant"));
            } else if (qName.matches("entry")) {
                key = attributes.getValue("key");
                cData = new StringBuffer();
            }
        }
        public void characters(char[] ch,
                int start,
                int length) {
            if ( message != null && key != null && length > 0 ) {
                cData.append(ch, start, length);
            }
        }
        
        public void endElement(String namespaceUri, String localeName, String qName) {
            if (qName.matches("locale")) {
                messages.put(message.getKey(), message);
            } else if (qName.matches("entry")) {
                message.addEntry(key, cData.toString());
                key = null;
            }
        }
        
        Map getMessages() {
            return messages;
        }
    }

    static class Message {
        private String id, language, country, variant;
        private Map entries = new HashMap();

        public Message(String id) {
            this.id = id;
        }

        public void addEntry(String key, String value) {
            entries.put(key, value);
        }

        public String getEntry(String key) {
            return (String)entries.get(key);
        }

        public Map getEntries() {
            return entries;
        }

        public void setLanguage(String language) {
            this.language = language;
        }

        public void setCountry(String country) {
            this.country = country;
        }

        public void setVariant(String variant) {
            this.variant = variant;
        }

        public String getKey() {
            StringBuffer key = new StringBuffer(64);
            key.append(id);
            if (language != null) key.append("_" + language);
            if (country != null) key.append("_" + country);
            if (variant != null) key.append("_" + variant);
            return key.toString();
        }
    }
}